
/*

     Primitives for reading a qcd data file written in a format suitable for
     reading by FORTRAN.

*/

#include <stdio.h>
#include <fcntl.h>
#include "fstrmhdr.h"
#include "spnhdr.h"

extern int  errno, sys_nerr ;
extern char *sys_errlist [] ;

static void sys_assert (condition, usr_message)
/*

  Test `condition'.  Abort program, reporting `usr_message', if it fails.

*/
int condition ;
char *usr_message ;
{
  if (!condition)
  {
    fprintf (stderr, "ERROR: %s", usr_message) ;
    if ((errno > 0) && (errno < sys_nerr))
      fprintf (stderr, "... %s", sys_errlist [errno]) ;
    fprintf (stderr, ".\n") ;
    exit (1) ;
  }
}

FDATA *fdata_open_read (file)
char *file ;
{
  FDATA *result ;

  result = (FDATA *) malloc (sizeof (FDATA)) ;

  if (file == NULL)
    result->desc = 0 ;
  else
  {
    char err_mess [128] ;

    result->desc = open (file, O_RDONLY, 0x1a4) ;

    strcpy (err_mess, "Failed to open \"") ;
    strcat (err_mess, file) ;
    strcat (err_mess, "\"") ;
    sys_assert (result->desc >= 0, err_mess) ;
  }

  return result ;
}

void fdata_close (data)
FDATA *data ;
{
  int result ;

  result   = read (data->desc, data->buf, FBUFFER_SIZE) ;
  if (result != 0)
    fprintf (stderr, "warning: file closed before eof encountered\n") ;

  close (data->desc) ;
  free  (data) ;
}

static void read_buffer (data, len_to_read)
FDATA *data ;
int len_to_read ;
{
  int ptr, residue ;

  ptr     = 0 ;
  residue = len_to_read ;
  while (residue > 0)
  {
    int result ;

    result   = read (data->desc, (data->buf) + ptr, residue) ;
    sys_assert (result != 0, "Unexpected EOF") ;
    sys_assert (result > 0, "Read failed") ;
    ptr     += result ;
    residue -= result ;
  }

}

void fdata_read_header (data, params, coeffs)
FDATA *data ;
int params [] ;
float coeffs [] ;
{
  float *buf ;
  int i, ptr ;
  int num_params, num_coeffs ;

  read_buffer (data, FBUFFER_SIZE) ;

  buf = (float *) data->buf ;

  ptr = 1 ;

  num_params = (int) buf [ptr] ;
  ptr++ ;

  for (i = 0; i < num_params; i++)
  {
    params [i] = (int) buf [ptr] ;
    ptr++ ;
  }

  num_coeffs = (int) buf [ptr] ;
  ptr++ ;

  for (i = 0; i < num_coeffs; i++)
  {
    coeffs [i] = buf [ptr] ;
    ptr++ ;
  }
}

void fdata_read_field_begin (data)
FDATA *data ;
{
  data->site_size        = sizeof (float) * N_SPIN * N_COL * 2 ;
  data->buf_ptr          = FBUFFER_SIZE ;
}

void fdata_read_field_record (data, field_record)
FDATA *data ;
float *field_record ;
{
  int cur_rec_space = FBUFFER_SIZE - data->buf_ptr ;
  int residue       = data->site_size - cur_rec_space ;

  if (residue > 0)
  {
    if (cur_rec_space > 0)
      memcpy ((char *) field_record, data->buf + data->buf_ptr, cur_rec_space) ;
    read_buffer (data, FBUFFER_SIZE) ;
    memcpy (((char *) field_record) + cur_rec_space, data->buf, residue) ;
    data->buf_ptr = residue ;
  }
  else
  {
    memcpy ((char *) field_record, data->buf + data->buf_ptr, data->site_size) ;
    data->buf_ptr += data->site_size ;
  }
}

